<?php

namespace Drupal\tooltip_taxonomy\Services;

use Drupal\Component\Plugin\Factory\FactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\ckeditor_vocabulary_list\Plugin\CKEditorPlugin\Vocabulary;

/**
 * Tooltip filter condition manager class
 * @author Mingsong Hu
 *        
 */
class TooltipManager {
  
  /**
   * Drupal Condition plugin for path.
   *
   * @var \Drupal\Core\Condition\ConditionPluginBase
   */
  protected $pathCondition;
  
  /**
   * Drupal condition plugin for content type.
   *
   * @var \Drupal\Core\Condition\ConditionPluginBase
   */
  protected $contentTypeCondition;
  
  /**
   * The field type manager.
   *
   * @var \Drupal\tooltip_taxonomy\Services\FieldTypeManager
   */
  protected $fieldTypeManager;
  
  /**
   * The entity type manager.
   * 
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * All conditions.
   * 
   * @var array
   */
  protected $allConditions = [];
  
  /**
   * The render service.
   * 
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;
  
  /**
   * Constructs an TooltipConditionManager object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entityTypeManager.
   */
  public function __construct(FactoryInterface $plugin_factory, FieldTypeManager $field_type_manager, EntityTypeManagerInterface $entity_type_manager, RendererInterface $renderer) {
    $this->pathCondition = $plugin_factory->createInstance('request_path');
    $this->contentTypeCondition = $plugin_factory->createInstance('node_type');
    $this->fieldTypeManager = $field_type_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->renderer = $renderer;
  }
  
  /**
   * Check the path and content type,
   * then return all conditions that
   * match current path and the content
   * types restrictions.
   * 
   * @param \Drupal\node\Entity\Node $node
   * 
   * @return array[]
   */
  public function checkPathAndContentType($node) {
    $all_con = $this->getAllFilterCondition();
    $match_cons = [];
    
    if (!empty($all_con)) {
      $this->allConditions = $all_con;
      // Check the path & content types.
      foreach ($all_con as $con) {
        $path_con = $con->get('path');
        if (!empty($path_con)) {
          $this->pathCondition->setConfiguration($path_con);
          if ($this->pathCondition->evaluate()) {
            $content_type_con = $con->get('contentTypes');
            
            if (!empty($content_type_con))
            {
              $this->contentTypeCondition->setConfiguration($content_type_con)
                ->setContextValue('node', $node);
              if ($this->contentTypeCondition->evaluate()) {
                $match_cons[] = $con;
              }
            }
          }      
        }
      }
    }
    
    return $match_cons;
  }
  
  /**
   * Get all tooltip filter conditions.
   * 
   * @return array|\Drupal\Core\Entity\EntityInterface[]
   */
  protected function getAllFilterCondition() {
    // Load the next release node.
    $cids = \Drupal::entityQuery('filter_condition')
 //   ->exists('vids')
    ->sort('weight')
    ->execute();
    
    if (empty($cids)) {
      return [];
    }
    
    // Load all nodes matched the conditions.
    return $this->entityTypeManager->getStorage('filter_condition')->loadMultiple($cids);
  }
  
  /**
   * Attach tooltip into a field text.
   * 
   * @param \Drupal\Core\Entity\EntityInterface $entity_type
   *   The entity type.
   *   
   * @param string $field_name
   *   The field machine name.
   *   
   * @param string $value
   *   The string needed to add tooltip into.
   *   
   * @return string
   */
  public function addTooltip(string $view_mode, \Drupal\Core\Entity\EntityInterface $entity, string $field_name, string $text, array &$tag) {
    $match_cons = $this->checkPathAndContentType($entity);
    // Taxonomy terms replacement array.
    $pattern_map = [];
    // Search keywords.
    $pattern_map['search'] = array();
    // Replace array.
    $pattern_map['replace'] = array();
    // The number of tooltips added.
    $count = 0;
    
    foreach ($match_cons as $con) {
      $allowed_views = $con->get('view');
      if (!empty($allowed_views)) {
        $not_all = false;
        foreach ($allowed_views as $mode) {
          if ($mode !== '0') {
            $not_all = true;
            break;
          }
        }
        
        if ($not_all && !in_array($view_mode, $allowed_views)) {
          // The current view mode doesn't match this condition.
          continue;
        }
      }
      
      $allowed_fields = $con->get('field');
      // Is there any selected field within this condition?
      if (!empty($allowed_fields)) {
        $field_key = $entity->getEntityTypeId() . '-' . $field_name;
        // Check if the field is a selected field in this condition.
        if (in_array($field_key, $allowed_fields)) {
          // Add the taxonomy terms from this condition into the pattern array.
          $this->addVocabularyReplacement($con->get('vids'), $pattern_map);
          // Cache tag.
          $tag[] = 'tooltip_taxonomy:' . $con->id();
        }
      }
      // No selected fields, all field applied.
      else {
        // Add the taxonomy terms from this condition into the pattern array.
        $this->addVocabularyReplacement($con->get('vids'), $pattern_map);
        // Cache tag.
        $tag[] = 'tooltip_taxonomy:' . $con->id();
      }
    }
    // Replace the taxonomy terms with tooltip markup.
    if (count($pattern_map['search']) > 0) {
      $newValue = preg_replace($pattern_map['search'], $pattern_map['replace'], $text, -1, $count);
    }
    else {
      return '';
    }
    
    return $newValue;
  }
  
  /**
   * Check if an vocabulary is used for tooltip.
   * 
   * @param string $vid
   * 
   * @return array
   *   All conditions' ID in which the Vocabulary
   *   is used for tooltip.
   */
  public function hasTooltip(string $vid) {
    $all_cons = $this->getAllFilterCondition();
    $result  = [];
    
    foreach ($all_cons as $con) {
      if (in_array($vid, $con->get('vids'))) {
        $result[] = $con->id();
      }
    }
    
    return $result;
  }
  
  /**
   * Add new taxonomy term pattern into the replacement array,
   * that will be used to replace the field text with tooltip markup.
   * 
   * @param array $vids
   *   Vocabulary array.
   * @param array $pattern_map
   *   replacement pattern array.
   * @return number
   *   The number of taxonomy terms added.
   */
  protected function addVocabularyReplacement(array $vids, array &$pattern_map) {
    $count = 0;
    
    if (empty($vids)) {
      return 0;
    }
    
    foreach ($vids as $vid) {
      $term_tree = $this->loadAllTaxonomyTerms($vid);
      if (!empty($term_tree)) {
        foreach ($term_tree as $term) {
          $des = strip_tags($term->description__value, "<b><i><strong><span><br>");
          if (empty($des)) {
            continue;
          }
          $tooltip_render = [
              '#theme' => 'tooltip_taxonomy',
              '#tooltip_id' => $term->vid . '-' . $term->tid,
              '#term_name' => $term->name,
              '#description' => $des,
          ];
          $name_pattern = '/\b' . $term->name . '\b/';
          // Check if there is a same term name.
          $name_key = array_search($name_pattern, $pattern_map['search']);
          if ($name_key === FALSE) {
            // New term.
            $pattern_map['search'][] = $name_pattern;
            $pattern_map['replace'][] =  $this->renderer->render($tooltip_render);
          }
          else {
            // Overwrite existing term.
            // Conditions that has bigger weight value
            // will overwrite the same term name.
            $pattern_map['replace'][$name_key] = $this->renderer->render($tooltip_render);
          }
          $count++;
        }
      }
    }
    
    return $count;
  }
  
  /**
   * Load all taxonomy terms of a vocabulary.
   *
   * @param string $entityType
   *   The type of the entity.
   *
   * @param string $bundleID
   *   The bundle ID.
   *
   * @return array
   *   All entities in the entity bundle.
   */
  protected function loadAllTaxonomyTerms(string $vid) {
    return $this->entityTypeManager->getStorage('taxonomy_term')->loadTree($vid);
  }
}

